@@ -41,9 +41,14 @@
 #include <spa/utils/ringbuffer.h>
 #include <spa/utils/string.h>
 
-#define NAME "loop"
+static struct spa_log_topic log_topic = SPA_LOG_TOPIC(0, "spa.loop");
+#undef SPA_LOG_TOPIC_DEFAULT
+#define SPA_LOG_TOPIC_DEFAULT &log_topic
 
-#define DATAS_SIZE (4096 * 8)
+#define MAX_ALIGN	8
+#define ITEM_ALIGN	8
+#define DATAS_SIZE	(4096*8)
+#define MAX_EP		32
 
 /** \cond */
 
@@ -75,15 +80,17 @@ struct impl {
 
 	int poll_fd;
 	pthread_t thread;
+	int enter_count;
 
 	struct spa_source *wakeup;
 	int ack_fd;
 
 	struct spa_ringbuffer buffer;
 	uint8_t *buffer_data;
-	uint8_t buffer_mem[DATAS_SIZE + 8];
+	uint8_t buffer_mem[DATAS_SIZE + MAX_ALIGN];
 
 	unsigned int flushing:1;
+	unsigned int polling:1;
 };
 
 struct source_impl {
@@ -92,7 +99,6 @@ struct source_impl {
 	struct impl *impl;
 	struct spa_list link;
 
-	bool close;
 	union {
 		spa_source_io_func_t io;
 		spa_source_idle_func_t idle;
@@ -100,8 +106,11 @@ struct source_impl {
 		spa_source_timer_func_t timer;
 		spa_source_signal_func_t signal;
 	} func;
-	bool enabled;
+
 	struct spa_source *fallback;
+
+	bool close;
+	bool enabled;
 };
 /** \endcond */
 
@@ -109,20 +118,50 @@ static int loop_add_source(void *object, struct spa_source *source)
 {
 	struct impl *impl = object;
 	source->loop = &impl->loop;
+	source->priv = NULL;
+	source->rmask = 0;
 	return spa_system_pollfd_add(impl->system, impl->poll_fd, source->fd, source->mask, source);
 }
 
 static int loop_update_source(void *object, struct spa_source *source)
 {
 	struct impl *impl = object;
+
+	spa_assert(source->loop == &impl->loop);
+
 	return spa_system_pollfd_mod(impl->system, impl->poll_fd, source->fd, source->mask, source);
 }
 
+static void detach_source(struct spa_source *source)
+{
+	struct spa_poll_event *e;
+
+	source->loop = NULL;
+	source->rmask = 0;
+
+	if ((e = source->priv)) {
+		/* active in an iteration of the loop, remove it from there */
+		e->data = NULL;
+		source->priv = NULL;
+	}
+}
+
+static int remove_from_poll(struct impl *impl, struct spa_source *source)
+{
+	spa_assert(source->loop == &impl->loop);
+
+	return spa_system_pollfd_del(impl->system, impl->poll_fd, source->fd);
+}
+
 static int loop_remove_source(void *object, struct spa_source *source)
 {
 	struct impl *impl = object;
-	source->loop = NULL;
-	return spa_system_pollfd_del(impl->system, impl->poll_fd, source->fd);
+	spa_assert(!impl->polling);
+
+	int res = remove_from_poll(impl, source);
+	detach_source(source);
+
+	return res;
 }
 
 static void flush_items(struct impl *impl)
@@ -138,7 +177,7 @@ static void flush_items(struct impl *impl)
 		item = SPA_PTROFF(impl->buffer_data, index & (DATAS_SIZE - 1), struct invoke_item);
 		block = item->block;
 
-		spa_log_trace(impl->log, NAME " %p: flush item %p", impl, item);
+		spa_log_trace_fp(impl->log, "%p: flush item %p", impl, item);
 		item->res = item->func ? item->func(&impl->loop,
 				true, item->seq, item->data, item->size,
 			   item->user_data) : 0;
@@ -147,7 +186,7 @@ static void flush_items(struct impl *impl)
 
 		if (block) {
 			if ((res = spa_system_eventfd_write(impl->system, impl->ack_fd, 1)) < 0)
-				spa_log_warn(impl->log, NAME " %p: failed to write event fd: %s",
+				spa_log_warn(impl->log, "%p: failed to write event fd: %s",
 						impl, spa_strerror(res));
 		}
 	}
@@ -183,17 +222,17 @@ loop_invoke(void *object,
 	int32_t filled;
 	uint32_t avail, idx, offset, l0;
 
-	if (pthread_equal(impl->thread, pthread_self()))
+	if (impl->thread == 0 || pthread_equal(impl->thread, pthread_self()))
 		return loop_invoke_inthread(impl, func, seq, data, size, block, user_data);
 
 	filled = spa_ringbuffer_get_write_index(&impl->buffer, &idx);
 	if (filled < 0 || filled > DATAS_SIZE) {
-		spa_log_warn(impl->log, NAME " %p: queue xrun %d", impl, filled);
+		spa_log_warn(impl->log, "%p: queue xrun %d", impl, filled);
 		return -EPIPE;
 	}
 	avail = DATAS_SIZE - filled;
 	if (avail < sizeof(struct invoke_item)) {
-		spa_log_warn(impl->log, NAME " %p: queue full %d", impl, avail);
+		spa_log_warn(impl->log, "%p: queue full %d", impl, avail);
 		return -EPIPE;
 	}
 	offset = idx & (DATAS_SIZE - 1);
@@ -208,9 +247,9 @@ loop_invoke(void *object,
 	item->size = size;
 	item->block = block;
 	item->user_data = user_data;
-	item->item_size = SPA_ROUND_UP_N(sizeof(struct invoke_item) + size, 8);
+	item->item_size = SPA_ROUND_UP_N(sizeof(struct invoke_item) + size, ITEM_ALIGN);
 
-	spa_log_trace(impl->log, NAME " %p: add item %p filled:%d", impl, item, filled);
+	spa_log_trace_fp(impl->log, "%p: add item %p filled:%d", impl, item, filled);
 
 	if (l0 >= item->item_size) {
 		/* item + size fit in current ringbuffer idx */
@@ -224,10 +263,10 @@ loop_invoke(void *object,
 		/* item does not fit, place the invoke_item at idx and start the
 		 * data at the start of the ringbuffer */
 		item->data = impl->buffer_data;
-		item->item_size = SPA_ROUND_UP_N(l0 + size, 8);
+		item->item_size = SPA_ROUND_UP_N(l0 + size, ITEM_ALIGN);
 	}
 	if (avail < item->item_size) {
-		spa_log_warn(impl->log, NAME " %p: queue full %d, need %zd", impl, avail,
+		spa_log_warn(impl->log, "%p: queue full %d, need %zd", impl, avail,
 				item->item_size);
 		return -EPIPE;
 	}
@@ -244,7 +283,7 @@ loop_invoke(void *object,
 		spa_loop_control_hook_before(&impl->hooks_list);
 
 		if ((res = spa_system_eventfd_read(impl->system, impl->ack_fd, &count)) < 0)
-			spa_log_warn(impl->log, NAME " %p: failed to read event fd: %s",
+			spa_log_warn(impl->log, "%p: failed to read event fd: %s",
 					impl, spa_strerror(res));
 
 		spa_loop_control_hook_after(&impl->hooks_list);
@@ -285,64 +324,122 @@ loop_add_hook(void *object,
 static void loop_enter(void *object)
 {
 	struct impl *impl = object;
-	impl->thread = pthread_self();
-	spa_log_trace(impl->log, NAME" %p: enter %lu", impl, impl->thread);
+	pthread_t thread_id = pthread_self();
+
+	if (impl->enter_count == 0) {
+		spa_return_if_fail(impl->thread == 0);
+		impl->thread = thread_id;
+		impl->enter_count = 1;
+	} else {
+		spa_return_if_fail(impl->enter_count > 0);
+		spa_return_if_fail(impl->thread == thread_id);
+		impl->enter_count++;
+	}
+	spa_log_trace(impl->log, "%p: enter %lu", impl, impl->thread);
 }
 
 static void loop_leave(void *object)
 {
 	struct impl *impl = object;
-	spa_log_trace(impl->log, NAME" %p: leave %lu", impl, impl->thread);
-	impl->thread = 0;
+	pthread_t thread_id = pthread_self();
+
+	spa_return_if_fail(impl->enter_count > 0);
+	spa_return_if_fail(impl->thread == thread_id);
+
+	spa_log_trace(impl->log, "%p: leave %lu", impl, impl->thread);
+
+	if (--impl->enter_count == 0) {
+		impl->thread = 0;
+		impl->polling = false;
+	}
+}
+
+static inline void free_source(struct source_impl *s)
+{
+	detach_source(&s->source);
+	free(s);
 }
 
 static inline void process_destroy(struct impl *impl)
 {
 	struct source_impl *source, *tmp;
+
 	spa_list_for_each_safe(source, tmp, &impl->destroy_list, link)
-		free(source);
+		free_source(source);
+
 	spa_list_init(&impl->destroy_list);
 }
 
+struct cancellation_handler_data {
+	struct spa_poll_event *ep;
+	int ep_count;
+};
+
+static void cancellation_handler(void *closure)
+{
+	const struct cancellation_handler_data *data = closure;
+
+	for (int i = 0; i < data->ep_count; i++) {
+		struct spa_source *s = data->ep[i].data;
+		if (SPA_LIKELY(s)) {
+			s->rmask = 0;
+			s->priv = NULL;
+		}
+	}
+}
+
 static int loop_iterate(void *object, int timeout)
 {
 	struct impl *impl = object;
-	struct spa_loop *loop = &impl->loop;
-	struct spa_poll_event ep[32];
+	struct spa_poll_event ep[MAX_EP], *e;
 	int i, nfds;
 
+	impl->polling = true;
 	spa_loop_control_hook_before(&impl->hooks_list);
 
 	nfds = spa_system_pollfd_wait(impl->system, impl->poll_fd, ep, SPA_N_ELEMENTS(ep), timeout);
 
 	spa_loop_control_hook_after(&impl->hooks_list);
+	impl->polling = false;
 
-	if (SPA_UNLIKELY(nfds < 0))
-		return nfds;
+	struct cancellation_handler_data cdata = { ep, nfds };
+	pthread_cleanup_push(cancellation_handler, &cdata);
 
 	/* first we set all the rmasks, then call the callbacks. The reason is that
 	 * some callback might also want to look at other sources it manages and
 	 * can then reset the rmask to suppress the callback */
 	for (i = 0; i < nfds; i++) {
 		struct spa_source *s = ep[i].data;
+
+		spa_assert(s->loop == &impl->loop);
+
 		s->rmask = ep[i].events;
+		/* already active in another iteration of the loop,
+		 * remove it from that iteration */
+		if (SPA_UNLIKELY(e = s->priv))
+			e->data = NULL;
+		s->priv = &ep[i];
 	}
-	for (i = 0; i < nfds; i++) {
-		struct spa_source *s = ep[i].data;
-		if (SPA_LIKELY(s->rmask && s->fd != -1 && s->loop == loop))
-			s->func(s);
-	}
+
 	if (SPA_UNLIKELY(!spa_list_is_empty(&impl->destroy_list)))
 		process_destroy(impl);
 
+	for (i = 0; i < nfds; i++) {
+		struct spa_source *s = ep[i].data;
+		if (SPA_LIKELY(s && s->rmask))
+			s->func(s);
+	}
+
+	pthread_cleanup_pop(true);
+
 	return nfds;
 }
 
 static void source_io_func(struct spa_source *source)
 {
-	struct source_impl *impl = SPA_CONTAINER_OF(source, struct source_impl, source);
-	spa_log_trace_fp(impl->impl->log, NAME" %p: io %08x", impl, source->rmask);
-	impl->func.io(source->data, source->fd, source->rmask);
+	struct source_impl *s = SPA_CONTAINER_OF(source, struct source_impl, source);
+	spa_log_trace_fp(s->impl->log, "%p: io %08x", s, source->rmask);
+	s->func.io(source->data, source->fd, source->rmask);
 }
 
 static struct spa_source *loop_add_io(void *object,
@@ -358,7 +455,6 @@ static struct spa_source *loop_add_io(void *object,
 	if (source == NULL)
 		goto error_exit;
 
-	source->source.loop = &impl->loop;
 	source->source.func = source_io_func;
 	source->source.data = data;
 	source->source.fd = fd;
@@ -378,7 +474,7 @@ static struct spa_source *loop_add_io(void *object,
 		source->fallback = spa_loop_utils_add_idle(&impl->utils,
 				mask & (SPA_IO_IN | SPA_IO_OUT) ? true : false,
 				(spa_source_idle_func_t) source_io_func, source);
-		spa_log_trace(impl->log, NAME" %p: adding fallback %p", impl,
+		spa_log_trace(impl->log, "%p: adding fallback %p", impl,
 				source->fallback);
 	}
 
@@ -398,8 +494,13 @@ static int loop_update_io(void *object, struct spa_source *source, uint32_t mask
 	struct impl *impl = object;
 	struct source_impl *s = SPA_CONTAINER_OF(source, struct source_impl, source);
 	int res;
+
+	spa_assert(s->impl == object);
+	spa_assert(source->func == source_io_func);
+
+	spa_log_trace(impl->log, "%p: update %08x -> %08x", s, source->mask, mask);
 	source->mask = mask;
-	spa_log_trace(impl->log, NAME" %p: update %08x", s, mask);
+
 	if (s->fallback)
 		res = spa_loop_utils_enable_idle(&impl->utils, s->fallback,
 				mask & (SPA_IO_IN | SPA_IO_OUT) ? true : false);
@@ -410,26 +511,29 @@ static int loop_update_io(void *object, struct spa_source *source, uint32_t mask
 
 static void source_idle_func(struct spa_source *source)
 {
-	struct source_impl *impl = SPA_CONTAINER_OF(source, struct source_impl, source);
-	impl->func.idle(source->data);
+	struct source_impl *s = SPA_CONTAINER_OF(source, struct source_impl, source);
+	s->func.idle(source->data);
 }
 
 static int loop_enable_idle(void *object, struct spa_source *source, bool enabled)
 {
-	struct source_impl *impl = SPA_CONTAINER_OF(source, struct source_impl, source);
+	struct source_impl *s = SPA_CONTAINER_OF(source, struct source_impl, source);
 	int res = 0;
 
-	if (enabled && !impl->enabled) {
-		if ((res = spa_system_eventfd_write(impl->impl->system, source->fd, 1)) < 0)
-			spa_log_warn(impl->impl->log, NAME " %p: failed to write idle fd %d: %s",
+	spa_assert(s->impl == object);
+	spa_assert(source->func == source_idle_func);
+
+	if (enabled && !s->enabled) {
+		if ((res = spa_system_eventfd_write(s->impl->system, source->fd, 1)) < 0)
+			spa_log_warn(s->impl->log, "%p: failed to write idle fd %d: %s",
 					source, source->fd, spa_strerror(res));
-	} else if (!enabled && impl->enabled) {
+	} else if (!enabled && s->enabled) {
 		uint64_t count;
-		if ((res = spa_system_eventfd_read(impl->impl->system, source->fd, &count)) < 0)
-			spa_log_warn(impl->impl->log, NAME " %p: failed to read idle fd %d: %s",
+		if ((res = spa_system_eventfd_read(s->impl->system, source->fd, &count)) < 0)
+			spa_log_warn(s->impl->log, "%p: failed to read idle fd %d: %s",
 					source, source->fd, spa_strerror(res));
 	}
-	impl->enabled = enabled;
+	s->enabled = enabled;
 	return res;
 }
 
@@ -447,7 +551,6 @@ static struct spa_source *loop_add_idle(void *object,
 	if ((res = spa_system_eventfd_create(impl->system, SPA_FD_CLOEXEC | SPA_FD_NONBLOCK)) < 0)
 		goto error_exit_free;
 
-	source->source.loop = &impl->loop;
 	source->source.func = source_idle_func;
 	source->source.data = data;
 	source->source.fd = res;
@@ -477,15 +580,15 @@ error_exit:
 
 static void source_event_func(struct spa_source *source)
 {
-	struct source_impl *impl = SPA_CONTAINER_OF(source, struct source_impl, source);
+	struct source_impl *s = SPA_CONTAINER_OF(source, struct source_impl, source);
 	uint64_t count = 0;
 	int res;
 
-	if ((res = spa_system_eventfd_read(impl->impl->system, source->fd, &count)) < 0)
-		spa_log_warn(impl->impl->log, NAME " %p: failed to read event fd %d: %s",
+	if ((res = spa_system_eventfd_read(s->impl->system, source->fd, &count)) < 0)
+		spa_log_warn(s->impl->log, "%p: failed to read event fd %d: %s",
 				source, source->fd, spa_strerror(res));
 
-	impl->func.event(source->data, count);
+	s->func.event(source->data, count);
 }
 
 static struct spa_source *loop_add_event(void *object,
@@ -502,7 +605,6 @@ static struct spa_source *loop_add_event(void *object,
 	if ((res = spa_system_eventfd_create(impl->system, SPA_FD_CLOEXEC | SPA_FD_NONBLOCK)) < 0)
 		goto error_exit_free;
 
-	source->source.loop = &impl->loop;
 	source->source.func = source_event_func;
 	source->source.data = data;
 	source->source.fd = res;
@@ -529,27 +631,30 @@ error_exit:
 
 static int loop_signal_event(void *object, struct spa_source *source)
 {
-	struct source_impl *impl = SPA_CONTAINER_OF(source, struct source_impl, source);
+	struct source_impl *s = SPA_CONTAINER_OF(source, struct source_impl, source);
 	int res;
 
-	if (SPA_UNLIKELY((res = spa_system_eventfd_write(impl->impl->system, source->fd, 1)) < 0))
-		spa_log_warn(impl->impl->log, NAME " %p: failed to write event fd %d: %s",
+	spa_assert(s->impl == object);
+	spa_assert(source->func == source_event_func);
+
+	if (SPA_UNLIKELY((res = spa_system_eventfd_write(s->impl->system, source->fd, 1)) < 0))
+		spa_log_warn(s->impl->log, "%p: failed to write event fd %d: %s",
 				source, source->fd, spa_strerror(res));
 	return res;
 }
 
 static void source_timer_func(struct spa_source *source)
 {
-	struct source_impl *impl = SPA_CONTAINER_OF(source, struct source_impl, source);
+	struct source_impl *s = SPA_CONTAINER_OF(source, struct source_impl, source);
 	uint64_t expirations = 0;
 	int res;
 
-	if (SPA_UNLIKELY((res = spa_system_timerfd_read(impl->impl->system,
+	if (SPA_UNLIKELY((res = spa_system_timerfd_read(s->impl->system,
 				source->fd, &expirations)) < 0))
-		spa_log_warn(impl->impl->log, NAME " %p: failed to read timer fd %d: %s",
+		spa_log_warn(s->impl->log, "%p: failed to read timer fd %d: %s",
 				source, source->fd, spa_strerror(res));
 
-	impl->func.timer(source->data, expirations);
+	s->func.timer(source->data, expirations);
 }
 
 static struct spa_source *loop_add_timer(void *object,
@@ -567,7 +672,6 @@ static struct spa_source *loop_add_timer(void *object,
 			SPA_FD_CLOEXEC | SPA_FD_NONBLOCK)) < 0)
 		goto error_exit_free;
 
-	source->source.loop = &impl->loop;
 	source->source.func = source_timer_func;
 	source->source.data = data;
 	source->source.fd = res;
@@ -596,10 +700,13 @@ static int
 loop_update_timer(void *object, struct spa_source *source,
 		  struct timespec *value, struct timespec *interval, bool absolute)
 {
-	struct impl *impl = object;
+	struct source_impl *s = SPA_CONTAINER_OF(source, struct source_impl, source);
 	struct itimerspec its;
 	int flags = 0, res;
 
+	spa_assert(s->impl == object);
+	spa_assert(source->func == source_timer_func);
+
 	spa_zero(its);
 	if (SPA_LIKELY(value)) {
 		its.it_value = *value;
@@ -612,7 +719,7 @@ loop_update_timer(void *object, struct spa_source *source,
 	if (SPA_LIKELY(absolute))
 		flags |= SPA_FD_TIMER_ABSTIME;
 
-	if (SPA_UNLIKELY((res = spa_system_timerfd_settime(impl->system, source->fd, flags, &its, NULL)) < 0))
+	if (SPA_UNLIKELY((res = spa_system_timerfd_settime(s->impl->system, source->fd, flags, &its, NULL)) < 0))
 		return res;
 
 	return 0;
@@ -620,14 +727,14 @@ loop_update_timer(void *object, struct spa_source *source,
 
 static void source_signal_func(struct spa_source *source)
 {
-	struct source_impl *impl = SPA_CONTAINER_OF(source, struct source_impl, source);
+	struct source_impl *s = SPA_CONTAINER_OF(source, struct source_impl, source);
 	int res, signal_number = 0;
 
-	if ((res = spa_system_signalfd_read(impl->impl->system, source->fd, &signal_number)) < 0)
-		spa_log_warn(impl->impl->log, NAME " %p: failed to read signal fd %d: %s",
+	if ((res = spa_system_signalfd_read(s->impl->system, source->fd, &signal_number)) < 0)
+		spa_log_warn(s->impl->log, "%p: failed to read signal fd %d: %s",
 				source, source->fd, spa_strerror(res));
 
-	impl->func.signal(source->data, signal_number);
+	s->func.signal(source->data, signal_number);
 }
 
 static struct spa_source *loop_add_signal(void *object,
@@ -646,7 +753,6 @@ static struct spa_source *loop_add_signal(void *object,
 			signal_number, SPA_FD_CLOEXEC | SPA_FD_NONBLOCK)) < 0)
 		goto error_exit_free;
 
-	source->source.loop = &impl->loop;
 	source->source.func = source_signal_func;
 	source->source.data = data;
 	source->source.fd = res;
@@ -673,22 +779,28 @@ error_exit:
 
 static void loop_destroy_source(void *object, struct spa_source *source)
 {
-	struct source_impl *impl = SPA_CONTAINER_OF(source, struct source_impl, source);
+	struct source_impl *s = SPA_CONTAINER_OF(source, struct source_impl, source);
 
-	spa_log_trace(impl->impl->log, NAME" %p ", impl);
+	spa_assert(s->impl == object);
 
-	spa_list_remove(&impl->link);
+	spa_log_trace(s->impl->log, "%p ", s);
 
-	if (impl->fallback)
-		loop_destroy_source(impl->impl, impl->fallback);
-	else if (source->loop)
-		loop_remove_source(impl->impl, source);
+	spa_list_remove(&s->link);
 
-	if (source->fd != -1 && impl->close) {
-		spa_system_close(impl->impl->system, source->fd);
+	if (s->fallback)
+		loop_destroy_source(s->impl, s->fallback);
+	else
+		remove_from_poll(s->impl, source);
+
+	if (source->fd != -1 && s->close) {
+		spa_system_close(s->impl->system, source->fd);
 		source->fd = -1;
 	}
-	spa_list_insert(&impl->impl->destroy_list, &impl->link);
+
+	if (!s->impl->polling)
+		free_source(s);
+	else
+		spa_list_insert(&s->impl->destroy_list, &s->link);
 }
 
 static const struct spa_loop_methods impl_loop = {
@@ -752,11 +864,15 @@ static int impl_clear(struct spa_handle *handle)
 
 	impl = (struct impl *) handle;
 
+	if (impl->enter_count != 0)
+		spa_log_warn(impl->log, "%p: loop is entered %d times",
+				impl, impl->enter_count);
+
+	spa_assert(!impl->polling);
+
 	spa_list_consume(source, &impl->source_list, link)
 		loop_destroy_source(impl, &source->source);
 
-	process_destroy(impl);
-
 	spa_system_close(impl->system, impl->ack_fd);
 	spa_system_close(impl->system, impl->poll_fd);
 
@@ -801,16 +917,17 @@ impl_init(const struct spa_handle_factory *factory,
 			&impl_loop_utils, impl);
 
 	impl->log = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Log);
+	spa_log_topic_init(impl->log, &log_topic);
 	impl->system = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_System);
 
 	if (impl->system == NULL) {
-		spa_log_error(impl->log, NAME " %p: a System is needed", impl);
+		spa_log_error(impl->log, "%p: a System is needed", impl);
 		res = -EINVAL;
 		goto error_exit;
 	}
 
 	if ((res = spa_system_pollfd_create(impl->system, SPA_FD_CLOEXEC)) < 0) {
-		spa_log_error(impl->log, NAME " %p: can't create pollfd: %s",
+		spa_log_error(impl->log, "%p: can't create pollfd: %s",
 				impl, spa_strerror(res));
 		goto error_exit;
 	}
@@ -820,24 +937,24 @@ impl_init(const struct spa_handle_factory *factory,
 	spa_list_init(&impl->destroy_list);
 	spa_hook_list_init(&impl->hooks_list);
 
-	impl->buffer_data = SPA_PTR_ALIGN(impl->buffer_mem, 8, uint8_t);
+	impl->buffer_data = SPA_PTR_ALIGN(impl->buffer_mem, MAX_ALIGN, uint8_t);
 	spa_ringbuffer_init(&impl->buffer);
 
 	impl->wakeup = loop_add_event(impl, wakeup_func, impl);
 	if (impl->wakeup == NULL) {
 		res = -errno;
-		spa_log_error(impl->log, NAME " %p: can't create wakeup event: %m", impl);
+		spa_log_error(impl->log, "%p: can't create wakeup event: %m", impl);
 		goto error_exit_free_poll;
 	}
 	if ((res = spa_system_eventfd_create(impl->system,
 			SPA_FD_EVENT_SEMAPHORE | SPA_FD_CLOEXEC)) < 0) {
-		spa_log_error(impl->log, NAME " %p: can't create ack event: %s",
+		spa_log_error(impl->log, "%p: can't create ack event: %s",
 				impl, spa_strerror(res));
 		goto error_exit_free_wakeup;
 	}
 	impl->ack_fd = res;
 
-	spa_log_debug(impl->log, NAME " %p: initialized", impl);
+	spa_log_debug(impl->log, "%p: initialized", impl);
 
 	return 0;
 
