@@ -124,18 +124,20 @@ static int try_load_conf(struct pw_context *this, const char *conf_prefix,
 static int context_set_freewheel(struct pw_context *context, bool freewheel)
 {
 	struct spa_thread *thr;
-	int res;
+	int res = 0;
 
 	if ((thr = pw_data_loop_get_thread(context->data_loop_impl)) == NULL)
 		return -EIO;
 
 	if (freewheel) {
 		pw_log_info("%p: enter freewheel", context);
-		res = pw_thread_utils_drop_rt(thr);
+		if (context->thread_utils)
+			res = spa_thread_utils_drop_rt(context->thread_utils, thr);
 	} else {
 		pw_log_info("%p: exit freewheel", context);
-		// Use the priority as configured within the realtime module
-		res = pw_thread_utils_acquire_rt(thr, -1);
+		/* Use the priority as configured within the realtime module */
+		if (context->thread_utils)
+			res = spa_thread_utils_acquire_rt(context->thread_utils, thr, -1);
 	}
 	if (res < 0)
 		pw_log_info("%p: freewheel error:%s", context, spa_strerror(res));
@@ -178,6 +180,22 @@ static void init_plugin_loader(struct impl *impl)
 		impl);
 }
 
+static int do_data_loop_setup(struct spa_loop *loop, bool async, uint32_t seq,
+		const void *data, size_t size, void *user_data)
+{
+	struct pw_context *this = user_data;
+	const char *str;
+	struct spa_cpu *cpu;
+
+	cpu = spa_support_find(this->support, this->n_support, SPA_TYPE_INTERFACE_CPU);
+
+	if ((str = pw_properties_get(this->properties, SPA_KEY_CPU_ZERO_DENORMALS)) != NULL &&
+	    cpu != NULL) {
+		pw_log_info("setting zero denormals: %s", str);
+		spa_cpu_zero_denormals(cpu, spa_atob(str));
+	}
+	return 0;
+}
 
 /** Create a new context object
  *
@@ -290,9 +308,6 @@ struct pw_context *pw_context_new(struct pw_loop *main_loop,
 		if (pw_properties_get(properties, PW_KEY_CPU_MAX_ALIGN) == NULL)
 			pw_properties_setf(properties, PW_KEY_CPU_MAX_ALIGN,
 				"%u", spa_cpu_get_max_align(cpu));
-		if ((str = pw_properties_get(properties, SPA_KEY_CPU_ZERO_DENORMALS)) == NULL)
-			str = "true";
-		spa_cpu_zero_denormals(cpu, spa_atob(str));
 	}
 
 	if (getenv("PIPEWIRE_DEBUG") == NULL &&
@@ -330,6 +345,12 @@ struct pw_context *pw_context_new(struct pw_loop *main_loop,
 	this->data_system = this->data_loop->system;
 	this->main_loop = main_loop;
 
+	this->work_queue = pw_work_queue_new(this->main_loop);
+	if (this->work_queue == NULL) {
+		res = -errno;
+		goto error_free;
+	}
+
 	init_plugin_loader(impl);
 
 	this->support[n_support++] = SPA_SUPPORT_INIT(SPA_TYPE_INTERFACE_System, this->main_loop->system);
@@ -389,7 +410,8 @@ struct pw_context *pw_context_new(struct pw_loop *main_loop,
 	if ((res = pw_data_loop_start(this->data_loop_impl)) < 0)
 		goto error_free;
 
-	context_set_freewheel(this, false);
+	pw_data_loop_invoke(this->data_loop_impl,
+			do_data_loop_setup, 0, NULL, 0, false, this);
 
 	pw_settings_expose(this);
 
@@ -442,7 +464,7 @@ void pw_context_destroy(struct pw_context *context)
 		pw_resource_destroy(resource);
 
 	if (context->data_loop_impl)
-		pw_data_loop_destroy(context->data_loop_impl);
+		pw_data_loop_stop(context->data_loop_impl);
 
 	spa_list_consume(module, &context->module_list, link)
 		pw_impl_module_destroy(module);
@@ -459,6 +481,9 @@ void pw_context_destroy(struct pw_context *context)
 	pw_log_debug("%p: free", context);
 	pw_context_emit_free(context);
 
+	if (context->data_loop_impl)
+		pw_data_loop_destroy(context->data_loop_impl);
+
 	if (context->pool)
 		pw_mempool_destroy(context->pool);
 
@@ -517,11 +542,15 @@ struct pw_loop *pw_context_get_main_loop(struct pw_context *context)
 	return context->main_loop;
 }
 
+SPA_EXPORT
+struct pw_data_loop *pw_context_get_data_loop(struct pw_context *context)
+{
+	return context->data_loop_impl;
+}
+
 SPA_EXPORT
 struct pw_work_queue *pw_context_get_work_queue(struct pw_context *context)
 {
-	if (context->work_queue == NULL)
-		context->work_queue = pw_work_queue_new(context->main_loop);
 	return context->work_queue;
 }
 
@@ -892,27 +921,15 @@ static int ensure_state(struct pw_impl_node *node, bool running)
 	return pw_impl_node_set_state(node, state);
 }
 
-static int collect_nodes(struct pw_context *context, struct pw_impl_node *node)
+static int collect_nodes(struct pw_context *context, struct pw_impl_node *node, struct spa_list *collect)
 {
 	struct spa_list queue;
-	struct pw_impl_node *n, *t, *driver;
+	struct pw_impl_node *n, *t;
 	struct pw_impl_port *p;
 	struct pw_impl_link *l;
 
 	pw_log_debug("node %p: '%s'", node, node->name);
 
-	if (node->driver) {
-		driver = node;
-		spa_list_consume(t, &driver->follower_list, follower_link) {
-			spa_list_remove(&t->follower_link);
-			spa_list_init(&t->follower_link);
-		}
-	} else {
-		driver = node->driver_node;
-		if (driver == NULL)
-			return -EINVAL;
-	}
-
 	/* start with node in the queue */
 	spa_list_init(&queue);
 	spa_list_append(&queue, &node->sort_link);
@@ -921,10 +938,15 @@ static int collect_nodes(struct pw_context *context, struct pw_impl_node *node)
 	/* now follow all the links from the nodes in the queue
 	 * and add the peers to the queue. */
 	spa_list_consume(n, &queue, sort_link) {
+		pw_log_debug(" next node %p: '%s'", n, n->name);
+
 		spa_list_remove(&n->sort_link);
-		pw_impl_node_set_driver(n, driver);
+		spa_list_append(collect, &n->sort_link);
 		n->passive = true;
 
+		if (!n->active)
+			continue;
+
 		spa_list_for_each(p, &n->input_ports, link) {
 			spa_list_for_each(l, &p->links, input_link) {
 				t = l->output->node;
@@ -938,7 +960,7 @@ static int collect_nodes(struct pw_context *context, struct pw_impl_node *node)
 					continue;
 
 				if (!l->passive)
-					driver->passive = n->passive = false;
+					node->passive = n->passive = false;
 
 				if (!t->visited) {
 					t->visited = true;
@@ -959,7 +981,7 @@ static int collect_nodes(struct pw_context *context, struct pw_impl_node *node)
 					continue;
 
 				if (!l->passive)
-					driver->passive = n->passive = false;
+					node->passive = n->passive = false;
 
 				if (!t->visited) {
 					t->visited = true;
@@ -985,6 +1007,27 @@ static int collect_nodes(struct pw_context *context, struct pw_impl_node *node)
 	return 0;
 }
 
+static void move_to_driver(struct pw_context *context, struct spa_list *nodes,
+		struct pw_impl_node *driver)
+{
+	struct pw_impl_node *n;
+	pw_log_debug("driver: %p %s", driver, driver->name);
+	spa_list_consume(n, nodes, sort_link) {
+		spa_list_remove(&n->sort_link);
+		pw_log_debug(" follower: %p %s", n, n->name);
+		pw_impl_node_set_driver(n, driver);
+	}
+}
+static void remove_from_driver(struct pw_context *context, struct spa_list *nodes)
+{
+	struct pw_impl_node *n;
+	spa_list_consume(n, nodes, sort_link) {
+		spa_list_remove(&n->sort_link);
+		pw_impl_node_set_driver(n, NULL);
+		ensure_state(n, false);
+	}
+}
+
 static inline void get_quantums(struct pw_context *context, uint32_t *def,
 		uint32_t *min, uint32_t *max, uint32_t *limit, uint32_t *rate)
 {
@@ -1061,6 +1104,24 @@ static bool rates_contains(uint32_t *rates, uint32_t n_rates, uint32_t rate)
 	return false;
 }
 
+/* here we evaluate the complete state of the graph.
+ *
+ * It roughly operates in 3 stages:
+ *
+ * 1. go over all drivers and collect the nodes that need to be scheduled with the
+ *    driver. This include all nodes that have an active link with the driver or
+ *    with a node already scheduled with the driver.
+ *
+ * 2. go over all nodes that are not assigned to a driver. The ones that require
+ *    a driver are moved to some random active driver found in step 1.
+ *
+ * 3. go over all drivers again, collect the quantum/rate of all followers, select
+ *    the desired final value and activate the followers and then the driver.
+ *
+ * A complete graph evaluation is performed for each change that is made to the
+ * graph, such as making/destroying links, adding/removing nodes, property changes such
+ * as quantum/rate changes or metadata changes.
+ */
 int pw_context_recalc_graph(struct pw_context *context, const char *reason)
 {
 	struct impl *impl = SPA_CONTAINER_OF(context, struct impl, this);
@@ -1068,7 +1129,8 @@ int pw_context_recalc_graph(struct pw_context *context, const char *reason)
 	struct pw_impl_node *n, *s, *target, *fallback;
 	uint32_t max_quantum, min_quantum, def_quantum, lim_quantum, rate_quantum;
 	uint32_t *rates, n_rates, def_rate;
-	bool freewheel = false, global_force_rate, force_rate, global_force_quantum;
+	bool freewheel = false, global_force_rate, force_rate, force_quantum, global_force_quantum;
+	struct spa_list collect;
 
 	pw_log_info("%p: busy:%d reason:%s", context, impl->recalc, reason);
 
@@ -1083,7 +1145,7 @@ again:
 	get_quantums(context, &def_quantum, &min_quantum, &max_quantum, &lim_quantum, &rate_quantum);
 	rates = get_rates(context, &def_rate, &n_rates, &global_force_rate);
 
-	global_force_quantum = rate_quantum == 0;
+	force_quantum = global_force_quantum = rate_quantum == 0;
 	force_rate = global_force_rate;
 
 	/* start from all drivers and group all nodes that are linked
@@ -1096,9 +1158,11 @@ again:
 		if (n->exported)
 			continue;
 
-		if (!n->visited)
-			collect_nodes(context, n);
-
+		if (!n->visited) {
+			spa_list_init(&collect);
+			collect_nodes(context, n, &collect);
+			move_to_driver(context, &collect, n);
+		}
 		/* from now on we are only interested in active driving nodes.
 		 * We're going to see if there are active followers. */
 		if (!n->driving || !n->active)
@@ -1135,29 +1199,41 @@ again:
 
 	/* now go through all available nodes. The ones we didn't visit
 	 * in collect_nodes() are not linked to any driver. We assign them
-	 * to either an active driver of the first driver */
+	 * to either an active driver or the first driver if they are in a
+	 * group that needs a driver. Else we remove them from a driver
+	 * and stop them. */
 	spa_list_for_each(n, &context->node_list, link) {
-		if (n->exported)
+		struct pw_impl_node *t, *driver;
+
+		if (n->exported || n->visited)
 			continue;
 
-		if (!n->visited) {
-			struct pw_impl_node *t;
+		pw_log_debug("%p: unassigned node %p: '%s' active:%d want_driver:%d target:%p",
+				context, n, n->name, n->active, n->want_driver, target);
 
-			pw_log_debug("%p: unassigned node %p: '%s' active:%d want_driver:%d target:%p",
-					context, n, n->name, n->active, n->want_driver, target);
+		/* collect all nodes in this group */
+		spa_list_init(&collect);
+		collect_nodes(context, n, &collect);
 
-			t = n->want_driver ? target : NULL;
-
-			pw_impl_node_set_driver(n, t);
-			if (t == NULL)
-				ensure_state(n, false);
-			else {
-				t->passive = false;
-				collect_nodes(context, n);
-			}
+		driver = NULL;
+		spa_list_for_each(t, &collect, sort_link) {
+			/* is any active and want a driver or it want process */
+			if ((t->want_driver && t->active && !n->passive) ||
+			    t->always_process)
+				driver = target;
+		}
+		if (driver != NULL) {
+			/* driver needed for this group */
+			driver->passive = false;
+			move_to_driver(context, &collect, driver);
+		} else {
+			/* no driver, make sure the nodes stops */
+			remove_from_driver(context, &collect);
 		}
-		n->visited = false;
 	}
+	/* clean up the visited flag now */
+	spa_list_for_each(n, &context->node_list, link)
+		n->visited = false;
 
 	/* assign final quantum and set state for followers and drivers */
 	spa_list_for_each(n, &context->driver_list, driver_link) {
@@ -1174,7 +1250,11 @@ again:
 		/* collect quantum and rate */
 		spa_list_for_each(s, &n->follower_list, follower_link) {
 
-			if (s->info.state > PW_NODE_STATE_SUSPENDED) {
+			if (!s->moved) {
+				/* We only try to enforce the lock flags for nodes that
+				 * are not recently moved between drivers. The nodes that
+				 * are moved should try to enforce their quantum on the
+				 * new driver. */
 				lock_quantum |= s->lock_quantum;
 				lock_rate |= s->lock_rate;
 			}
@@ -1183,6 +1263,7 @@ again:
 				def_quantum = min_quantum = max_quantum = s->force_quantum;
 				rate_quantum = 0;
 				quantum_stamp = s->stamp;
+				force_quantum = true;
 			}
 			if (!global_force_rate && s->force_rate > 0 &&
 			    s->stamp > rate_stamp) {
@@ -1211,8 +1292,12 @@ again:
 
 			if (s->active)
 				running = !n->passive;
+
+			s->moved = false;
 		}
 
+		if (force_quantum)
+			lock_quantum = false;
 		if (force_rate)
 			lock_rate = false;
 
@@ -1220,9 +1305,14 @@ again:
 		if (lock_rate ||
 		    (!force_rate &&
 		    (n->info.state > PW_NODE_STATE_IDLE)))
+			/* when someone wants us to lock the rate of this driver or
+			 * when the driver is busy and we don't need to force a rate,
+			 * keep the current rate */
 			target_rate = current_rate;
 		else {
-			/* calculate desired rate */
+			/* Here we are allowed to change the rate of the driver.
+			 * Start with the default rate. If the desired rate is
+			 * allowed, switch to it */
 			target_rate = def_rate;
 			if (rate.denom != 0 && rate.num == 1) {
 				if (rates_contains(rates, n_rates, rate.denom))
@@ -1231,6 +1321,7 @@ again:
 		}
 
 		if (target_rate != current_rate) {
+			/* we doing a rate switch */
 			pw_log_info("(%s-%u) state:%s new rate:%u->%u",
 					n->name, n->info.id,
 					pw_node_state_as_string(n->info.state),
@@ -1244,6 +1335,8 @@ again:
 				if (n->info.state >= PW_NODE_STATE_IDLE)
 					suspend_driver(context, n);
 			}
+			/* we're setting the pending rate. This will become the new
+			 * current rate in the next iteration of the graph. */
 			n->current_rate = SPA_FRACTION(1, target_rate);
 			n->current_pending = true;
 			current_rate = target_rate;
@@ -1252,6 +1345,7 @@ again:
 		}
 
 		if (rate_quantum != 0 && current_rate != rate_quantum) {
+			/* the quantum values are scaled with the current rate */
 			def_quantum = def_quantum * current_rate / rate_quantum;
 			min_quantum = min_quantum * current_rate / rate_quantum;
 			max_quantum = max_quantum * current_rate / rate_quantum;
@@ -1278,11 +1372,16 @@ again:
 					n->name, n->info.id,
 					n->current_quantum,
 					quantum);
+			/* this is the new pending quantum */
 			n->current_quantum = quantum;
 			n->current_pending = true;
 		}
 
 		if (n->info.state < PW_NODE_STATE_RUNNING && n->current_pending) {
+			/* the driver node is not actually running and we have a
+			 * panding change. Apply the change to the position now so
+			 * that we have the right values when we change the node
+			 * states of the driver and followers to RUNNING below */
 			n->rt.position->clock.duration = n->current_quantum;
 			n->rt.position->clock.rate = n->current_rate;
 			n->current_pending = false;
@@ -1291,6 +1390,7 @@ again:
 		pw_log_debug("%p: driving %p running:%d passive:%d quantum:%u '%s'",
 				context, n, running, n->passive, quantum, n->name);
 
+		/* first change the node states of the followers to the new target */
 		spa_list_for_each(s, &n->follower_list, follower_link) {
 			if (s == n)
 				continue;
@@ -1298,6 +1398,7 @@ again:
 					context, s, s->active, s->name);
 			ensure_state(s, running);
 		}
+		/* now that all the followers are ready, start the driver */
 		ensure_state(n, running);
 	}
 	impl->recalc = false;
@@ -1433,6 +1534,12 @@ int pw_context_set_object(struct pw_context *context, const char *type, void *va
 		}
 		entry->value = value;
 	}
+	if (spa_streq(type, SPA_TYPE_INTERFACE_ThreadUtils)) {
+		context->thread_utils = value;
+		if (context->data_loop_impl)
+			pw_data_loop_set_thread_utils(context->data_loop_impl,
+					context->thread_utils);
+	}
 	return 0;
 }
 
