@@ -25,13 +25,15 @@
 #include <stdio.h>
 #include <signal.h>
 #include <getopt.h>
+#include <locale.h>
 
 #include <spa/utils/result.h>
+#include <spa/utils/string.h>
 #include <spa/pod/parser.h>
 #include <spa/debug/pod.h>
 
 #include <pipewire/impl.h>
-#include <extensions/profiler.h>
+#include <pipewire/extensions/profiler.h>
 
 #define MAX_NAME		128
 #define MAX_FOLLOWERS		64
@@ -88,17 +90,16 @@ struct point {
 
 static int process_info(struct data *d, const struct spa_pod *pod, struct point *point)
 {
-	spa_pod_parse_struct(pod,
+	return spa_pod_parse_struct(pod,
 			SPA_POD_Long(&point->count),
 			SPA_POD_Float(&point->cpu_load[0]),
 			SPA_POD_Float(&point->cpu_load[1]),
 			SPA_POD_Float(&point->cpu_load[2]));
-	return 0;
 }
 
 static int process_clock(struct data *d, const struct spa_pod *pod, struct point *point)
 {
-	spa_pod_parse_struct(pod,
+	return spa_pod_parse_struct(pod,
 			SPA_POD_Int(&point->clock.flags),
 			SPA_POD_Int(&point->clock.id),
 			SPA_POD_Stringn(point->clock.name, sizeof(point->clock.name)),
@@ -109,7 +110,6 @@ static int process_clock(struct data *d, const struct spa_pod *pod, struct point
 			SPA_POD_Long(&point->clock.delay),
 			SPA_POD_Double(&point->clock.rate_diff),
 			SPA_POD_Long(&point->clock.next_nsec));
-	return 0;
 }
 
 static int process_driver_block(struct data *d, const struct spa_pod *pod, struct point *point)
@@ -117,20 +117,22 @@ static int process_driver_block(struct data *d, const struct spa_pod *pod, struc
 	char *name = NULL;
 	uint32_t driver_id = 0;
 	struct measurement driver;
+	int res;
 
 	spa_zero(driver);
-	spa_pod_parse_struct(pod,
+	if ((res = spa_pod_parse_struct(pod,
 			SPA_POD_Int(&driver_id),
 			SPA_POD_String(&name),
 			SPA_POD_Long(&driver.prev_signal),
 			SPA_POD_Long(&driver.signal),
 			SPA_POD_Long(&driver.awake),
 			SPA_POD_Long(&driver.finish),
-			SPA_POD_Int(&driver.status));
+			SPA_POD_Int(&driver.status))) < 0)
+		return res;
 
 	if (d->driver_id == 0) {
 		d->driver_id = driver_id;
-		fprintf(stderr, "logging driver %u\n", driver_id);
+		printf("logging driver %u\n", driver_id);
 	}
 	else if (d->driver_id != driver_id)
 		return -1;
@@ -143,7 +145,7 @@ static int find_follower(struct data *d, uint32_t id, const char *name)
 {
 	int i;
 	for (i = 0; i < d->n_followers; i++) {
-		if (d->followers[i].id == id && strcmp(d->followers[i].name, name) == 0)
+		if (d->followers[i].id == id && spa_streq(d->followers[i].name, name))
 			return i;
 	}
 	return -1;
@@ -161,7 +163,7 @@ static int add_follower(struct data *d, uint32_t id, const char *name)
 	strncpy(d->followers[idx].name, name, MAX_NAME);
 	d->followers[idx].name[MAX_NAME-1] = '\0';
 	d->followers[idx].id = id;
-	fprintf(stderr, "logging follower %u (\"%s\")\n", id, name);
+	printf("logging follower %u (\"%s\")\n", id, name);
 
 	return idx;
 }
@@ -171,17 +173,18 @@ static int process_follower_block(struct data *d, const struct spa_pod *pod, str
 	uint32_t id = 0;
 	const char *name =  NULL;
 	struct measurement m;
-	int idx;
+	int res, idx;
 
 	spa_zero(m);
-	spa_pod_parse_struct(pod,
+	if ((res = spa_pod_parse_struct(pod,
 			SPA_POD_Int(&id),
 			SPA_POD_String(&name),
 			SPA_POD_Long(&m.prev_signal),
 			SPA_POD_Long(&m.signal),
 			SPA_POD_Long(&m.awake),
 			SPA_POD_Long(&m.finish),
-			SPA_POD_Int(&m.status));
+			SPA_POD_Int(&m.status))) < 0)
+		return res;
 
 	if ((idx = find_follower(d, id, name)) < 0) {
 		if ((idx = add_follower(d, id, name)) < 0) {
@@ -241,7 +244,7 @@ static void dump_point(struct data *d, struct point *point)
 		d->last_status = point->clock.nsec;
 	}
 	else if (point->clock.nsec - d->last_status > SPA_NSEC_PER_SEC) {
-		fprintf(stderr, "logging %"PRIi64" samples  %"PRIi64" seconds [CPU %f %f %f]\r",
+		printf("logging %"PRIi64" samples  %"PRIi64" seconds [CPU %f %f %f]\r",
 				d->count, (int64_t) ((d->last_status - d->start_status) / SPA_NSEC_PER_SEC),
 				point->cpu_load[0], point->cpu_load[1], point->cpu_load[2]);
 		d->last_status = point->clock.nsec;
@@ -257,7 +260,7 @@ static void dump_scripts(struct data *d)
 	if (d->driver_id == 0)
 		return;
 
-	fprintf(stderr, "\ndumping scripts for %d followers\n", d->n_followers);
+	printf("\ndumping scripts for %d followers\n", d->n_followers);
 
 	out = fopen("Timing1.plot", "w");
 	if (out == NULL) {
@@ -418,7 +421,7 @@ static void dump_scripts(struct data *d)
 			"gnuplot Timing5.plot\n");
 		fclose(out);
 	}
-	fprintf(stderr, "run 'sh generate_timings.sh' and load Timings.html in a browser\n");
+	printf("run 'sh generate_timings.sh' and load Timings.html in a browser\n");
 }
 
 static void profiler_profile(void *data, const struct spa_pod *pod)
@@ -473,7 +476,7 @@ static void registry_event_global(void *data, uint32_t id,
 	struct data *d = data;
 	struct pw_proxy *proxy;
 
-	if (strcmp(type, PW_TYPE_INTERFACE_Profiler) != 0)
+	if (!spa_streq(type, PW_TYPE_INTERFACE_Profiler))
 		return;
 
 	if (d->profiler != NULL) {
@@ -485,7 +488,7 @@ static void registry_event_global(void *data, uint32_t id,
 	if (proxy == NULL)
 		goto error_proxy;
 
-	fprintf(stderr, "Attaching to Profiler id:%d\n", id);
+	printf("Attaching to Profiler id:%d\n", id);
 	d->profiler = proxy;
 	pw_proxy_add_object_listener(proxy, &d->profiler_listener, &profiler_events, d);
 
@@ -537,9 +540,9 @@ static void do_quit(void *data, int signal_number)
 	pw_main_loop_quit(d->loop);
 }
 
-static void show_help(const char *name)
+static void show_help(const char *name, bool error)
 {
-        fprintf(stdout, "%s [options]\n"
+        fprintf(error ? stderr : stdout, "%s [options]\n"
 		"  -h, --help                            Show this help\n"
 		"      --version                         Show version\n"
 		"  -r, --remote                          Remote daemon name\n"
@@ -563,15 +566,16 @@ int main(int argc, char *argv[])
 	};
 	int c;
 
+	setlocale(LC_ALL, "");
 	pw_init(&argc, &argv);
 
 	while ((c = getopt_long(argc, argv, "hVr:o:", long_options, NULL)) != -1) {
 		switch (c) {
 		case 'h':
-			show_help(argv[0]);
+			show_help(argv[0], false);
 			return 0;
 		case 'V':
-			fprintf(stdout, "%s\n"
+			printf("%s\n"
 				"Compiled with libpipewire %s\n"
 				"Linked with libpipewire %s\n",
 				argv[0],
@@ -585,7 +589,7 @@ int main(int argc, char *argv[])
 			opt_remote = optarg;
 			break;
 		default:
-			show_help(argv[0]);
+			show_help(argv[0], true);
 			return -1;
 		}
 	}
@@ -626,7 +630,7 @@ int main(int argc, char *argv[])
 		return -1;
 	}
 
-	fprintf(stderr, "Logging to %s\n", data.filename);
+	printf("Logging to %s\n", data.filename);
 
 	pw_core_add_listener(data.core,
 				   &data.core_listener,
@@ -641,8 +645,13 @@ int main(int argc, char *argv[])
 
 	pw_main_loop_run(data.loop);
 
-	pw_proxy_destroy((struct pw_proxy*)data.profiler);
+	if (data.profiler) {
+		spa_hook_remove(&data.profiler_listener);
+		pw_proxy_destroy((struct pw_proxy*)data.profiler);
+	}
+	spa_hook_remove(&data.registry_listener);
 	pw_proxy_destroy((struct pw_proxy*)data.registry);
+	spa_hook_remove(&data.core_listener);
 	pw_context_destroy(data.context);
 	pw_main_loop_destroy(data.loop);
 
